@@ -25,17 +25,16 @@ These transformation tools form the backbone of the code optimization pipeline,
 maintaining semantic integrity and performance characteristics.
 """
 
-from astToolkit import (
-	Be, DOT, Grab, IfThis, IngredientsFunction, IngredientsModule, Make, NodeChanger, NodeTourist, Then, 木,
-)
-from autoflake import fix_code as autoflake_fix_code
+from astToolkit import Be, DOT, Grab, IfThis, Make, NodeChanger, NodeTourist, Then, 木
 from collections.abc import Mapping
 from copy import deepcopy
+from hunterMakesPy import raiseIfNone
+from hunterMakesPy.filesystemToolkit import settings_autoflakeDEFAULT, writePython
 from os import PathLike
 from pathlib import PurePath
 from typing import Any
-from Z0Z_tools import raiseIfNone, writeStringToHere
 import ast
+import io
 
 def makeDictionaryAsyncFunctionDef(astAST: ast.AST) -> dict[str, ast.AsyncFunctionDef]:
 	"""
@@ -128,48 +127,60 @@ def makeDictionaryMosDef(astAST: ast.AST) -> dict[str, ast.AsyncFunctionDef | as
 	dictionaryIdentifier2MosDef.update(makeDictionaryFunctionDef(astAST))
 	return dictionaryIdentifier2MosDef
 
-def inlineFunctionDef(identifierToInline: str, module: ast.Module) -> ast.FunctionDef:  # noqa: C901, PLR0912
+def inlineFunctionDef(identifierToInline: str, astModule: ast.Module) -> ast.FunctionDef:
 	"""
-	Inline function calls within a function definition to create a self-contained function.
+	Synthesize a new function from `identifierToInline` with the logic "inlined" from functions called by `identifierToInline`.
 
-	(AI generated docstring)
+	`inlineFunctionDef` starts with the `ast.FunctionDef` in `astModule` whose `ast.FunctionDef.name == identifierToInline`, then
+	`inlineFunctionDef` searches through `identifierToInline` for calls to functions defined in `astModule`.
 
-	This function takes a function identifier and a module, finds the function definition,
-	and then recursively inlines all function calls within that function with their
-	implementation bodies. This produces a fully inlined function that doesn't depend
-	on other function definitions from the module.
+	More specifically, `inlineFunctionDef` searches for an `ast.Call` to an `ast.Name` (*e.g.*, `Path`, but not the
+	`ast.Attribute`, `pathlib.Path`). The `ast.Name` identifier is in `ast.Name.id`, for example, `ast.Name.id = "Path"`.
+	`inlineFunctionDef` then searches in `astModule` for an `ast.FunctionDef.name` that is the same as the `ast.Name.id`. If
+	`inlineFunctionDef` finds a matching `ast.FunctionDef`, it will replace `ast.Call` with the logic from the called function.
+
+	`inlineFunctionDef` repeats the process until it gets bored or until it cannot find any more functions to inline. Therefore,
+	`inlineFunctionDef` will inline functions that were not called directly by `identifierToInline` in the original `astModule`.
+
+	`inlineFunctionDef`, however, does not inline an `ast.FunctionDef` if the `ast.FunctionDef` calls itself or if a "child" of
+	the `ast.FunctionDef` calls the original `ast.FunctionDef`.
 
 	Parameters
 	----------
 	identifierToInline : str
-		The name of the function to inline.
-	module : ast.Module
-		The AST module containing the function and its dependencies.
+		The identifier of the target function; the `str` must match an `ast.FunctionDef.name` in `astModule`.
+	astModule : ast.Module
+		(abstract syntax tree Module) An `ast.Module` with `ast.FunctionDef.name == identifierToInline` and zero or more other
+		`ast.FunctionDef` to inline.
 
 	Returns
 	-------
 	FunctionDefToInline : ast.FunctionDef
-		The inlined function definition as an `ast.FunctionDef` object.
+		The synthesized function with inlined logic from other functions defined in `astModule`.
 
 	Raises
 	------
-		ValueError: If the function to inline is not found in the module.
+		ValueError: If `identifierToInline` does not match any `ast.FunctionDef.name` in `astModule`.
 	"""
-	dictionaryFunctionDef: dict[str, ast.FunctionDef] = makeDictionaryFunctionDef(module)
+	dictionaryFunctionDef: dict[str, ast.FunctionDef] = makeDictionaryFunctionDef(astModule)
 	try:
 		FunctionDefToInline = dictionaryFunctionDef[identifierToInline]
 	except KeyError as ERRORmessage:
-		message = f"FunctionDefToInline not found in dictionaryIdentifier2FunctionDef: {identifierToInline = }"
+		message = f"I was unable to find an `ast.FunctionDef` with name {identifierToInline = } in {astModule = }."
 		raise ValueError(message) from ERRORmessage
 
 	listIdentifiersCalledFunctions: list[str] = []
+# TODO I probably have the skill now to expand this from only `IfThis.isCallToName` to include more options for `ast.Call.func`,
+# such as attribute and subscript access. `IfThis.isNestedNameIdentifier` might be perfect. BUT, I think this will affect how I
+# key `dictionaryFunctionDef`.
 	findIdentifiersToInline = NodeTourist[ast.Call, ast.expr](IfThis.isCallToName
 		, Grab.funcAttribute(Grab.idAttribute(Then.appendTo(listIdentifiersCalledFunctions))))
 	findIdentifiersToInline.visit(FunctionDefToInline)
 
 	dictionary4Inlining: dict[str, ast.FunctionDef] = {}
 	for identifier in sorted(set(listIdentifiersCalledFunctions).intersection(dictionaryFunctionDef.keys())):
-		if NodeTourist(IfThis.matchesMeButNotAnyDescendant(IfThis.isCallIdentifier(identifier)), Then.extractIt).captureLastMatch(module) is not None:
+# TODO Learn how real programmers avoid infinite loops but still inline recursive functions.
+		if NodeTourist(IfThis.matchesMeButNotAnyDescendant(IfThis.isCallIdentifier(identifier)), Then.extractIt).captureLastMatch(astModule) is not None:
 			dictionary4Inlining[identifier] = dictionaryFunctionDef[identifier]
 
 	keepGoing = True
@@ -182,7 +193,7 @@ def inlineFunctionDef(identifierToInline: str, module: ast.Module) -> ast.Functi
 		if len(listIdentifiersCalledFunctions) > 0:
 			keepGoing = True
 			for identifier in listIdentifiersCalledFunctions:
-				if NodeTourist(IfThis.matchesMeButNotAnyDescendant(IfThis.isCallIdentifier(identifier)), Then.extractIt).captureLastMatch(module) is not None:
+				if NodeTourist(IfThis.matchesMeButNotAnyDescendant(IfThis.isCallIdentifier(identifier)), Then.extractIt).captureLastMatch(astModule) is not None:
 					FunctionDefTarget = dictionaryFunctionDef[identifier]
 					if len(FunctionDefTarget.body) == 1:
 						replacement = NodeTourist(Be.Return, Then.extractIt(DOT.value)).captureLastMatch(FunctionDefTarget)
@@ -206,52 +217,104 @@ def inlineFunctionDef(identifierToInline: str, module: ast.Module) -> ast.Functi
 	ast.fix_missing_locations(FunctionDefToInline)
 	return FunctionDefToInline
 
-def removeUnusedParameters(ingredientsFunction: IngredientsFunction) -> IngredientsFunction:
+def pythonCode2ast_expr(string: str) -> ast.expr:
+	"""PROTOTYPE Convert *one* expression-as-a-string of Python code to an `ast.expr`.
+
+	TODO add list of applicable subclasses. Until then, see `Make.expr` for an approximate list.
+
+	Note well
+	---------
+	This prototype "shortcut" function has approximately 482 *implied* constraints and pitfalls. If you can't get it to do what
+	you want, I recommend saving yourself a bunch of stress and not using this shortcut.
 	"""
-	Remove unused parameters from a function's AST definition, return statement, and annotation.
+	return raiseIfNone(NodeTourist(Be.Expr, Then.extractIt(DOT.value)).captureLastMatch(ast.parse(string)))
 
-	(AI generated docstring)
+def removeUnusedParameters(FunctionDef: ast.FunctionDef) -> ast.FunctionDef:
+	"""
+	Remove unused parameters from `FunctionDef`.
 
-	This function analyzes the Abstract Syntax Tree (AST) of a given function and removes
-	any parameters that are not referenced within the function body. It updates the
-	function signature, the return statement (if it's a tuple containing unused variables),
-	and the return type annotation accordingly.
+	The `removeUnusedParameters` function removes `ast.arg` (abstract syntax tree ***arg***ument) parameters from the
+	`ast.arguments` argument specification of the `ast.FunctionDef` (Function ***Def***inition) function in
+	`FunctionDef` that are are not referenced in the `ast.FunctionDef.body` or that are only referenced in
+	`ast.Return` return statements.
+
+	This function will replace every `return` statement with a new `return` statement that returns all of the remaining
+	parameters. It will update the `returns` annotation.
 
 	Parameters
 	----------
-	ingredientsFunction : IngredientsFunction
+	FunctionDef : ast.FunctionDef
 		An object containing the AST representation of a function to be processed.
 
 	Returns
 	-------
-	IngredientsFunction : IngredientsFunction
-		The modified IngredientsFunction object with unused parameters and corresponding return
-		elements/annotations removed from its AST.
+	ast.FunctionDef : ast.FunctionDef
+		The modified `ast.FunctionDef` object with unused parameters and corresponding return elements/annotations removed from its AST.
 	"""
-	list_argCuzMyBrainRefusesToThink = ingredientsFunction.astFunctionDef.args.args + ingredientsFunction.astFunctionDef.args.posonlyargs + ingredientsFunction.astFunctionDef.args.kwonlyargs
+	list_argCuzMyBrainRefusesToThink = FunctionDef.args.args + FunctionDef.args.posonlyargs + FunctionDef.args.kwonlyargs
 	list_arg_arg: list[str] = [ast_arg.arg for ast_arg in list_argCuzMyBrainRefusesToThink]
 	listName: list[ast.Name] = []
-	fauxFunctionDef = deepcopy(ingredientsFunction.astFunctionDef)
+	fauxFunctionDef = deepcopy(FunctionDef)
 	NodeChanger(Be.Return, Then.removeIt).visit(fauxFunctionDef)
 	NodeTourist(Be.Name, Then.appendTo(listName)).visit(fauxFunctionDef)
 	listIdentifiers: list[str] = [astName.id for astName in listName]
 	listIdentifiersNotUsed: list[str] = list(set(list_arg_arg) - set(listIdentifiers))
 	for argIdentifier in listIdentifiersNotUsed:
 		remove_arg = NodeChanger(IfThis.is_argIdentifier(argIdentifier), Then.removeIt)
-		remove_arg.visit(ingredientsFunction.astFunctionDef)
+		remove_arg.visit(FunctionDef)
 
-	list_argCuzMyBrainRefusesToThink = ingredientsFunction.astFunctionDef.args.args + ingredientsFunction.astFunctionDef.args.posonlyargs + ingredientsFunction.astFunctionDef.args.kwonlyargs
+	list_argCuzMyBrainRefusesToThink = FunctionDef.args.args + FunctionDef.args.posonlyargs + FunctionDef.args.kwonlyargs
 
 	listName = [Make.Name(ast_arg.arg) for ast_arg in list_argCuzMyBrainRefusesToThink]
 	replaceReturn = NodeChanger(Be.Return, Then.replaceWith(Make.Return(Make.Tuple(listName))))
-	replaceReturn.visit(ingredientsFunction.astFunctionDef)
+	replaceReturn.visit(FunctionDef)
 
 	list_annotation: list[ast.expr] = [ast_arg.annotation for ast_arg in list_argCuzMyBrainRefusesToThink if ast_arg.annotation is not None]
-	ingredientsFunction.astFunctionDef.returns = Make.Subscript(Make.Name('tuple'), Make.Tuple(list_annotation))
+	FunctionDef.returns = Make.Subscript(Make.Name('tuple'), Make.Tuple(list_annotation))
 
-	ast.fix_missing_locations(ingredientsFunction.astFunctionDef)
+	ast.fix_missing_locations(FunctionDef)
 
-	return ingredientsFunction
+	return FunctionDef
+
+def unjoinBinOP(astAST: ast.AST, operator: type[ast.operator] = ast.operator) -> list[ast.expr]:
+	"""
+	Unjoin a binary operation AST node into a list of expressions.
+
+	(AI generated docstring)
+
+	This function takes an AST node representing a binary operation and recursively
+	unjoins it into a flat list of expressions. It handles both binary operations
+	and unary operations, ensuring that all nested expressions are extracted.
+
+	Parameters
+	----------
+	astAST : ast.AST
+		The AST node to unjoin.
+	operator : type[ast.operator] = ast.operator
+		The type of binary operator to look for in the AST. Defaults to `ast.operator`.
+
+	Returns
+	-------
+	list[ast.expr]
+		A list of expressions extracted from the binary operation AST node.
+	"""
+	list_ast_expr: list[ast.expr] = []
+	workbench: list[ast.expr] = []
+
+	findThis = Be.BinOp.opIs(lambda this_op: isinstance(this_op, operator))
+	doThat = Grab.andDoAllOf([Grab.leftAttribute(Then.appendTo(workbench)), Grab.rightAttribute(Then.appendTo(list_ast_expr))])
+	breakingBinOp = NodeTourist(findThis, doThat)
+
+	breakingBinOp.visit(astAST)
+
+	while workbench:
+		ast_expr = workbench.pop()
+		if isinstance(ast_expr, ast.BinOp):
+			breakingBinOp.visit(ast_expr)
+		else:
+			list_ast_expr.append(ast_expr)
+
+	return list_ast_expr
 
 def unparseFindReplace(astTree: 木, mappingFindReplaceNodes: Mapping[ast.AST, ast.AST]) -> 木:
 	"""
@@ -293,40 +356,38 @@ def unparseFindReplace(astTree: 木, mappingFindReplaceNodes: Mapping[ast.AST, a
 			astTree = deepcopy(newTree)
 	return newTree
 
-def write_astModule(ingredients: IngredientsModule, pathFilename: PathLike[Any] | PurePath, packageName: str | None = None) -> None:
+def write_astModule(astModule: ast.Module, pathFilename: PathLike[Any] | PurePath | io.TextIOBase, settings: dict[str, dict[str, Any]] | None = None, identifierPackage: str='') -> None:
 	"""
-	Convert an IngredientsModule to Python source code and write it to a file.
+	Convert an AST module to Python source code and write it to a file or stream.
 
 	(AI generated docstring)
 
-	This function renders an IngredientsModule into executable Python code,
-	applies code quality improvements like import organization via autoflake,
-	and writes the result to the specified file path.
+	This function takes an AST module structure, converts it to formatted Python source code,
+	and writes the result to a file or text stream. The function ensures all AST node locations
+	are properly fixed before conversion and applies code formatting and optimization through
+	configurable settings.
 
-	The function performs several key steps:
-	1. Converts the AST module structure to a valid Python AST
-	2. Fixes location attributes in the AST for proper formatting
-	3. Converts the AST to Python source code
-	4. Optimizes imports using autoflake
-	5. Writes the final source code to the specified file location
-
-	This is typically the final step in the code generation assembly line,
-	producing optimized Python modules ready for execution.
+	By default, the function uses `autoflake` to remove unused imports and variables, and `isort`
+	to organize import statements. Custom formatting settings can be provided to control this behavior.
 
 	Parameters
 	----------
-	ingredients : IngredientsModule
-		The IngredientsModule containing the module definition.
-	pathFilename : PathLike[Any] | PurePath
-		The file path where the module should be written.
-	packageName : str | None = None
-		Optional package name to preserve in import optimization.
+	astModule : ast.Module
+		The AST module to convert and write. This should be a complete module structure with
+		all necessary imports, statements, and definitions.
+	pathFilename : PathLike[Any] | PurePath | io.TextIOBase
+		The destination for the generated Python code. Can be a file path or an open text stream.
+	settings : dict[str, dict[str, Any]] | None = None
+		Optional configuration for code formatting tools. Should contain nested dictionaries with
+		keys like `'autoflake'` and `'isort'`, each mapping to their respective tool settings.
+	identifierPackage : str = ''
+		Optional package identifier to add to the autoflake additional imports list when no
+		settings are provided, ensuring the specified package is preserved during import optimization.
+
 	"""
-	astModule = Make.Module(ingredients.body, ingredients.type_ignores)
 	ast.fix_missing_locations(astModule)
-	pythonSource: str = raiseIfNone(ast.unparse(astModule))
-	autoflake_additional_imports: list[str] = ingredients.imports.exportListModuleIdentifiers()
-	if packageName:
-		autoflake_additional_imports.append(packageName)
-	pythonSource = autoflake_fix_code(pythonSource, autoflake_additional_imports, expand_star_imports=False, remove_all_unused_imports=True, remove_duplicate_keys = False, remove_unused_variables = False)
-	writeStringToHere(pythonSource, pathFilename)
+	pythonSource: str = ast.unparse(astModule)
+	if identifierPackage and not settings:
+		settings = {'autoflake': settings_autoflakeDEFAULT}
+		settings['autoflake']['additional_imports'].append(identifierPackage)
+	writePython(pythonSource, pathFilename, settings)
